
BEGIN {
  unless ($ENV{RELEASE_TESTING}) {
    require Test::More;
    Test::More::plan(skip_all => 'these tests are for release candidate testing');
  }
}

use strict;
use warnings;

# this test was generated with Dist::Zilla::Plugin::Test::NoTabs 0.07

use Test::More 0.88;
use Test::NoTabs;

my @files = (
    'lib/Elastic/Doc.pm',
    'lib/Elastic/Manual.pod',
    'lib/Elastic/Manual/Analysis.pod',
    'lib/Elastic/Manual/Attributes.pod',
    'lib/Elastic/Manual/Attributes/Unique.pod',
    'lib/Elastic/Manual/Intro.pod',
    'lib/Elastic/Manual/NoSQL.pod',
    'lib/Elastic/Manual/QueryDSL.pod',
    'lib/Elastic/Manual/QueryDSL/Filters.pod',
    'lib/Elastic/Manual/QueryDSL/Nested.pod',
    'lib/Elastic/Manual/QueryDSL/Queries.pod',
    'lib/Elastic/Manual/Reindex.pod',
    'lib/Elastic/Manual/Scaling.pod',
    'lib/Elastic/Manual/Scoping.pod',
    'lib/Elastic/Manual/Searching.pod',
    'lib/Elastic/Manual/Terminology.pod',
    'lib/Elastic/Model.pm',
    'lib/Elastic/Model/Alias.pm',
    'lib/Elastic/Model/Bulk.pm',
    'lib/Elastic/Model/Deleted.pm',
    'lib/Elastic/Model/Domain.pm',
    'lib/Elastic/Model/Index.pm',
    'lib/Elastic/Model/Meta/Class/Doc.pm',
    'lib/Elastic/Model/Meta/Class/Model.pm',
    'lib/Elastic/Model/Namespace.pm',
    'lib/Elastic/Model/Result.pm',
    'lib/Elastic/Model/Results.pm',
    'lib/Elastic/Model/Results/Cached.pm',
    'lib/Elastic/Model/Results/Scrolled.pm',
    'lib/Elastic/Model/Role/Doc.pm',
    'lib/Elastic/Model/Role/Index.pm',
    'lib/Elastic/Model/Role/Iterator.pm',
    'lib/Elastic/Model/Role/Model.pm',
    'lib/Elastic/Model/Role/Results.pm',
    'lib/Elastic/Model/Role/Store.pm',
    'lib/Elastic/Model/Scope.pm',
    'lib/Elastic/Model/SearchBuilder.pm',
    'lib/Elastic/Model/Store.pm',
    'lib/Elastic/Model/Trait/Exclude.pm',
    'lib/Elastic/Model/Trait/Field.pm',
    'lib/Elastic/Model/TypeMap/Base.pm',
    'lib/Elastic/Model/TypeMap/Common.pm',
    'lib/Elastic/Model/TypeMap/Default.pm',
    'lib/Elastic/Model/TypeMap/ES.pm',
    'lib/Elastic/Model/TypeMap/Moose.pm',
    'lib/Elastic/Model/TypeMap/Objects.pm',
    'lib/Elastic/Model/TypeMap/Structured.pm',
    'lib/Elastic/Model/Types.pm',
    'lib/Elastic/Model/UID.pm',
    'lib/Elastic/Model/View.pm',
    't/00_setup/02_load.t',
    't/00_setup/03_domain.t',
    't/00_setup/04_roles.t',
    't/10_typemaps/01_map_moose.t',
    't/10_typemaps/02_map_moosex.t',
    't/10_typemaps/03_map_structured.t',
    't/10_typemaps/04_map_objects.t',
    't/10_typemaps/05_map_common.t',
    't/10_typemaps/06_map_es.t',
    't/10_typemaps/11_flate_moose.t',
    't/10_typemaps/12_flate_moose_x.t',
    't/10_typemaps/13_flate_structured.t',
    't/10_typemaps/14_flate_objects.t',
    't/10_typemaps/15_flate_common.t',
    't/10_typemaps/16_flate_es.t',
    't/10_typemaps/21_string.t',
    't/10_typemaps/22_number.t',
    't/10_typemaps/23_date.t',
    't/10_typemaps/24_boolean.t',
    't/10_typemaps/25_binary.t',
    't/10_typemaps/26_object.t',
    't/10_typemaps/27_nested.t',
    't/10_typemaps/28_ip4.t',
    't/10_typemaps/29_geopoint.t',
    't/10_typemaps/40_index_config.t',
    't/10_typemaps/test_field.pl',
    't/10_typemaps/test_flation.pl',
    't/10_typemaps/test_mapping.pl',
    't/20_namespace/01_basic.t',
    't/20_namespace/02_index.t',
    't/20_namespace/03_alias.t',
    't/20_namespace/04_index_role.t',
    't/20_namespace/05_namespace_for_domain.t',
    't/30_domain/01_crud.t',
    't/30_domain/02_conflict.t',
    't/30_domain/03_uniques.t',
    't/30_domain/04_nested_has_changed.t',
    't/40_view/01_view_attrs.t',
    't/40_view/02_view_methods.t',
    't/40_view/03_results.t',
    't/40_view/04_scrolled_results.t',
    't/40_view/05_cached_results.t',
    't/40_view/06_result.t',
    't/40_view/07_routing.t',
    't/50_scope/01_get.t',
    't/50_scope/02_store.t',
    't/50_scope/03_delete.t',
    't/50_scope/04_search.t',
    't/60_reindex/01_basic.t',
    't/60_reindex/02_objects_with_uids.t',
    't/60_reindex/03_transform.t',
    't/65_bulk/01_bulk.t',
    't/65_bulk/02_error_handlers.t',
    't/65_bulk/03_scope.t',
    't/70_searchbuilder/00-basic.t',
    't/70_searchbuilder/01-filters-field-ops.t',
    't/70_searchbuilder/02-filters-unary-ops.t',
    't/70_searchbuilder/03-filters-logic.t',
    't/70_searchbuilder/04-queries-field-ops.t',
    't/70_searchbuilder/05-queries-unary-ops.t',
    't/70_searchbuilder/06-queries-logic.t',
    't/70_searchbuilder/07-raw_dsl.t',
    't/70_searchbuilder/08-misc.t',
    't/70_searchbuilder/10_object_filters.t',
    't/70_searchbuilder/11_object_queries.t',
    't/author-strict.t',
    't/lib/FieldTest.pm',
    't/lib/FieldTest/Binary.pm',
    't/lib/FieldTest/Boolean.pm',
    't/lib/FieldTest/Date.pm',
    't/lib/FieldTest/GeoPoint.pm',
    't/lib/FieldTest/IP4.pm',
    't/lib/FieldTest/Nested.pm',
    't/lib/FieldTest/Number.pm',
    't/lib/FieldTest/Object.pm',
    't/lib/FieldTest/String.pm',
    't/lib/Foo.pm',
    't/lib/Foo/Post.pm',
    't/lib/Foo/Post/Base.pm',
    't/lib/Foo/User.pm',
    't/lib/IndexConfig.pm',
    't/lib/IndexConfig/AllOpts.pm',
    't/lib/IndexConfig/BadAnalyzer.pm',
    't/lib/IndexConfig/BadTokenizer.pm',
    't/lib/IndexConfig/NoAnalyzer.pm',
    't/lib/MyApp.pm',
    't/lib/MyApp/MultiUser.pm',
    't/lib/MyApp/Post.pm',
    't/lib/MyApp/UniqUser.pm',
    't/lib/MyApp/User.pm',
    't/lib/MyAppUniq.pm',
    't/lib/Nested.pm',
    't/lib/RoleTest.pm',
    't/lib/RoleTest/Class.pm',
    't/lib/RoleTest/RoleFour.pm',
    't/lib/RoleTest/RoleOne.pm',
    't/lib/RoleTest/RoleThree.pm',
    't/lib/RoleTest/RoleTwo.pm',
    't/lib/TypeTest.pm',
    't/lib/TypeTest/Common.pm',
    't/lib/TypeTest/ES.pm',
    't/lib/TypeTest/Moose.pm',
    't/lib/TypeTest/MooseX.pm',
    't/lib/TypeTest/Objects.pm',
    't/lib/TypeTest/Structured.pm',
    't/lib/TypeTest/TypeMap.pm',
    't/lib/es.pl',
    't/release-eol.t',
    't/release-no-tabs.t',
    't/release-pod-syntax.t'
);

notabs_ok($_) foreach @files;
done_testing;
